@@ -6,50 +6,57 @@ use std::convert::TryFrom;
 use std::ops::Deref;
 
 #[derive(Debug, PartialEq)]
-struct OidStr {
-    oid: Vec<u64>,
-    data: String,
+pub struct OidStr {
+    pub oid: Vec<u64>,
+    pub data: String,
 }
 
 #[derive(Debug, PartialEq)]
-enum X509Name {
+pub enum X509Name {
     Utf8(OidStr),
     PrStr(OidStr),
+    TtxStr(OidStr),
+    Ia5Str(OidStr),
 }
 
 #[derive(Debug, PartialEq)]
-struct RsaPub {
-    sign_oid: Vec<u64>,
-    pub_oid: Vec<u64>,
-    n: Vec<u8>,
-    e: u32,
+pub enum X509Time {
+    Utc(i64),
+    Gen(i64),
 }
 
 #[derive(Debug, PartialEq)]
-struct EcPub {
-    sign_oid: Vec<u64>,
-    pub_oid: Vec<u64>,
-    key: Vec<u8>,
-    curve: Vec<u64>,
+pub struct RsaPub {
+    pub pub_oid: Vec<u64>,
+    pub n: Vec<u8>,
+    pub e: u32,
 }
 
 #[derive(Debug, PartialEq)]
-enum PubKey {
+pub struct EcPub {
+    pub pub_oid: Vec<u64>,
+    pub key: Vec<u8>,
+    pub curve: Vec<u64>,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum PubKey {
     Rsa(RsaPub),
     Ec(EcPub),
 }
 
 #[derive(Debug, PartialEq)]
 pub struct X509 {
-    version: Option<u64>,
-    sn: u64,
-    issuer: Vec<X509Name>,
-    subject: Vec<X509Name>,
-    not_before: i64,
-    not_after: i64,
-    pub_key: Option<PubKey>,
-    ext: Vec<X509Ext>,
-    sign: Vec<u8>,
+    pub version: Option<u64>,
+    pub sn: Vec<u8>,
+    pub issuer: Vec<X509Name>,
+    pub subject: Vec<X509Name>,
+    pub not_before: Option<X509Time>,
+    pub not_after: Option<X509Time>,
+    pub pub_key: Option<PubKey>,
+    pub ext: Vec<X509Ext>,
+    pub sign_oid: Vec<u64>,
+    pub sign: Vec<u8>,
 }
 
 fn serialize(a: &ASN1Block) -> Option<Vec<u8>> {
@@ -69,7 +76,7 @@ fn oid_new(id: &Vec<u64>) -> OID {
     OID::new(res)
 }
 
-fn seq_oid_utf8(v: &OidStr) -> Vec<ASN1Block> {
+fn seq_oid_str(v: &OidStr) -> Vec<ASN1Block> {
     let mut vec = Vec::new();
     vec.push(ASN1Block::ObjectIdentifier(0, oid_new(&v.oid)));
     vec.push(ASN1Block::PrintableString(1, String::from(&v.data)));
@@ -80,7 +87,7 @@ fn seq_oid_utf8(v: &OidStr) -> Vec<ASN1Block> {
     seq
 }
 
-fn seq_oid_str(v: &OidStr) -> Vec<ASN1Block> {
+fn seq_oid_utf8(v: &OidStr) -> Vec<ASN1Block> {
     let mut vec = Vec::new();
     vec.push(ASN1Block::ObjectIdentifier(0, oid_new(&v.oid)));
     vec.push(ASN1Block::UTF8String(1, String::from(&v.data)));
@@ -91,19 +98,59 @@ fn seq_oid_str(v: &OidStr) -> Vec<ASN1Block> {
     seq
 }
 
+fn seq_oid_ttx(v: &OidStr) -> Vec<ASN1Block> {
+    let mut vec = Vec::new();
+    vec.push(ASN1Block::ObjectIdentifier(0, oid_new(&v.oid)));
+    vec.push(ASN1Block::TeletexString(1, String::from(&v.data)));
+
+    let mut seq = Vec::new();
+    seq.push(ASN1Block::Sequence(0, vec));
+
+    seq
+}
+
+fn seq_oid_ia5(v: &OidStr) -> Vec<ASN1Block> {
+    let mut vec = Vec::new();
+    vec.push(ASN1Block::ObjectIdentifier(0, oid_new(&v.oid)));
+    vec.push(ASN1Block::IA5String(1, String::from(&v.data)));
+
+    let mut seq = Vec::new();
+    seq.push(ASN1Block::Sequence(0, vec));
+
+    seq
+}
+
 fn x509_name(name: &Vec<X509Name>) -> Vec<ASN1Block> {
     let mut n = Vec::new();
 
     for i in 0..name.len() {
         match &name[i] {
-            X509Name::Utf8(s) => n.push(ASN1Block::Set(0, seq_oid_str(s))),
-            X509Name::PrStr(s) => n.push(ASN1Block::Set(0, seq_oid_utf8(s))),
+            X509Name::Utf8(s) => n.push(ASN1Block::Set(0, seq_oid_utf8(s))),
+            X509Name::PrStr(s) => n.push(ASN1Block::Set(0, seq_oid_str(s))),
+            X509Name::TtxStr(s) => n.push(ASN1Block::Set(0, seq_oid_ttx(s))),
+            X509Name::Ia5Str(s) => n.push(ASN1Block::Set(0, seq_oid_ia5(s))),
         }
     }
 
     n
 }
 
+fn validity_time(not_before: &X509Time, not_after: &X509Time) -> Vec<ASN1Block> {
+    let mut v = Vec::new();
+
+    match not_before {
+        X509Time::Utc(nb) => v.push(ASN1Block::UTCTime(0, Utc.timestamp(*nb, 0))),
+        X509Time::Gen(nb) => v.push(ASN1Block::GeneralizedTime(0, Utc.timestamp(*nb, 0))),
+    }
+
+    match not_after {
+        X509Time::Utc(na) => v.push(ASN1Block::UTCTime(0, Utc.timestamp(*na, 0))),
+        X509Time::Gen(na) => v.push(ASN1Block::GeneralizedTime(0, Utc.timestamp(*na, 0))),
+    }
+
+    v
+}
+
 fn version_explicit(val: u64) -> ASN1Block {
     ASN1Block::Explicit(
         ASN1Class::ContextSpecific,
@@ -180,6 +227,14 @@ fn ec_pub(ec: &EcPub) -> Vec<ASN1Block> {
     ret
 }
 
+fn build_pub_key(pub_key: &Option<PubKey>) -> Option<Vec<ASN1Block>> {
+    match pub_key {
+        Some(PubKey::Rsa(ref rsa)) => rsa_pub(rsa),
+        Some(PubKey::Ec(ref ec)) => Some(ec_pub(ec)),
+        None => None,
+    }
+}
+
 fn x509_body(x: &X509) -> Option<Vec<ASN1Block>> {
     let mut body = Vec::new();
 
@@ -189,12 +244,12 @@ fn x509_body(x: &X509) -> Option<Vec<ASN1Block>> {
     }
 
     /* Serial Number */
-    body.push(ASN1Block::Integer(0, BigInt::from(x.sn)));
+    body.push(ASN1Block::Integer(0, BigInt::from_signed_bytes_be(&x.sn)));
 
     /* Signature Algorithm */
     match x.pub_key {
-        Some(PubKey::Rsa(ref rsa)) => body.push(ASN1Block::Sequence(0, null_oid(&rsa.sign_oid))),
-        Some(PubKey::Ec(ref ec)) => body.push(ASN1Block::Sequence(0, vec_oid(&ec.sign_oid))),
+        Some(PubKey::Rsa(_)) => body.push(ASN1Block::Sequence(0, null_oid(&x.sign_oid))),
+        Some(PubKey::Ec(_)) => body.push(ASN1Block::Sequence(0, vec_oid(&x.sign_oid))),
         None => {
             println!("Failed: no public key");
             return None;
@@ -210,9 +265,15 @@ fn x509_body(x: &X509) -> Option<Vec<ASN1Block>> {
     }
 
     /* Validity time */
-    let mut validity = Vec::new();
-    validity.push(ASN1Block::UTCTime(0, Utc.timestamp(x.not_before, 0)));
-    validity.push(ASN1Block::UTCTime(0, Utc.timestamp(x.not_after, 0)));
+    if x.not_before == None || x.not_after == None {
+        println!("Failed: no validity time");
+        return None;
+    }
+
+    let validity = validity_time(
+        &x.not_before.as_ref().unwrap(),
+        &x.not_after.as_ref().unwrap(),
+    );
     body.push(ASN1Block::Sequence(0, validity));
 
     /* Subject name */
@@ -224,15 +285,8 @@ fn x509_body(x: &X509) -> Option<Vec<ASN1Block>> {
     }
 
     /* Subject Public Key Info */
-    match x.pub_key {
-        Some(PubKey::Rsa(ref rsa)) => {
-            let rsa_vec = match rsa_pub(rsa) {
-                Some(r) => r,
-                None => return None,
-            };
-            body.push(ASN1Block::Sequence(0, rsa_vec));
-        }
-        Some(PubKey::Ec(ref ec)) => body.push(ASN1Block::Sequence(0, ec_pub(ec))),
+    match build_pub_key(&x.pub_key) {
+        Some(v) => body.push(ASN1Block::Sequence(0, v)),
         None => return None,
     }
 
@@ -287,6 +341,7 @@ impl X509 {
             not_after: self.not_after,
             pub_key: self.pub_key,
             ext: self.ext,
+            sign_oid: self.sign_oid,
             sign: sign,
         };
 
@@ -299,10 +354,8 @@ impl X509 {
 
         x509.push(ASN1Block::Sequence(0, body));
         match self.pub_key {
-            Some(PubKey::Rsa(ref rsa)) => {
-                x509.push(ASN1Block::Sequence(0, null_oid(&rsa.sign_oid)))
-            }
-            Some(PubKey::Ec(ref ec)) => x509.push(ASN1Block::Sequence(0, vec_oid(&ec.sign_oid))),
+            Some(PubKey::Rsa(_)) => x509.push(ASN1Block::Sequence(0, null_oid(&self.sign_oid))),
+            Some(PubKey::Ec(_)) => x509.push(ASN1Block::Sequence(0, vec_oid(&self.sign_oid))),
             None => return None,
         }
 
@@ -318,31 +371,48 @@ impl X509 {
 
         serialize(x509_full.first()?)
     }
+
+    pub fn pub_key(self) -> Option<Vec<u8>> {
+        let asn = match build_pub_key(&self.pub_key) {
+            Some(a) => a,
+            None => return None,
+        };
+
+        match simple_asn1::to_der(&ASN1Block::Sequence(0, asn)) {
+            Ok(der) => Some(der),
+            Err(_) => {
+                println!("Failed to serialize Public Key");
+                return None;
+            }
+        }
+    }
 }
 
 #[derive(Default)]
 pub struct X509Builder {
     version: Option<u64>,
-    sn: u64,
+    sn: Vec<u8>,
     issuer: Vec<X509Name>,
     subject: Vec<X509Name>,
-    not_before: i64,
-    not_after: i64,
+    not_before: Option<X509Time>,
+    not_after: Option<X509Time>,
     pub_key: Option<PubKey>,
     ext: Vec<X509Ext>,
+    sign_oid: Vec<u64>,
 }
 
 impl X509Builder {
-    pub fn new(sn: u64) -> X509Builder {
+    pub fn new(sn: Vec<u8>) -> X509Builder {
         X509Builder {
             version: None,
             sn: sn,
             issuer: Vec::new(),
             subject: Vec::new(),
-            not_before: 0,
-            not_after: 0,
+            not_before: None,
+            not_after: None,
             pub_key: None,
             ext: Vec::new(),
+            sign_oid: Vec::new(),
         }
     }
 
@@ -369,6 +439,24 @@ impl X509Builder {
         self
     }
 
+    pub fn issuer_ttxstr(mut self, oid: Vec<u64>, data: &str) -> X509Builder {
+        let name = X509Name::TtxStr(OidStr {
+            oid: oid,
+            data: data.to_string(),
+        });
+        self.issuer.push(name);
+        self
+    }
+
+    pub fn issuer_ia5str(mut self, oid: Vec<u64>, data: &str) -> X509Builder {
+        let name = X509Name::Ia5Str(OidStr {
+            oid: oid,
+            data: data.to_string(),
+        });
+        self.issuer.push(name);
+        self
+    }
+
     pub fn subject_utf8(mut self, oid: Vec<u64>, data: &str) -> X509Builder {
         let name = X509Name::Utf8(OidStr {
             oid: oid,
@@ -387,26 +475,47 @@ impl X509Builder {
         self
     }
 
-    pub fn not_before(mut self, not_before: i64) -> X509Builder {
-        self.not_before = not_before;
+    pub fn subject_ttxstr(mut self, oid: Vec<u64>, data: &str) -> X509Builder {
+        let name = X509Name::TtxStr(OidStr {
+            oid: oid,
+            data: data.to_string(),
+        });
+        self.subject.push(name);
         self
     }
 
-    pub fn not_after(mut self, not_after: i64) -> X509Builder {
-        self.not_after = not_after;
+    pub fn subject_ia5str(mut self, oid: Vec<u64>, data: &str) -> X509Builder {
+        let name = X509Name::Ia5Str(OidStr {
+            oid: oid,
+            data: data.to_string(),
+        });
+        self.subject.push(name);
         self
     }
 
-    pub fn pub_key_rsa(
-        mut self,
-        sign_oid: Vec<u64>,
-        pub_oid: Vec<u64>,
-        n: Vec<u8>,
-        e: u32,
-    ) -> X509Builder {
+    pub fn not_before_utc(mut self, not_before: i64) -> X509Builder {
+        self.not_before = Some(X509Time::Utc(not_before));
+        self
+    }
+
+    pub fn not_before_gen(mut self, not_before: i64) -> X509Builder {
+        self.not_before = Some(X509Time::Gen(not_before));
+        self
+    }
+
+    pub fn not_after_utc(mut self, not_after: i64) -> X509Builder {
+        self.not_after = Some(X509Time::Utc(not_after));
+        self
+    }
+
+    pub fn not_after_gen(mut self, not_after: i64) -> X509Builder {
+        self.not_after = Some(X509Time::Gen(not_after));
+        self
+    }
+
+    pub fn pub_key_rsa(mut self, oid: Vec<u64>, n: Vec<u8>, e: u32) -> X509Builder {
         let key = PubKey::Rsa(RsaPub {
-            sign_oid: sign_oid,
-            pub_oid: pub_oid,
+            pub_oid: oid,
             n: n,
             e: e,
         });
@@ -414,16 +523,9 @@ impl X509Builder {
         self
     }
 
-    pub fn pub_key_ec(
-        mut self,
-        sign_oid: Vec<u64>,
-        pub_oid: Vec<u64>,
-        key: Vec<u8>,
-        curve: Vec<u64>,
-    ) -> X509Builder {
+    pub fn pub_key_ec(mut self, oid: Vec<u64>, key: Vec<u8>, curve: Vec<u64>) -> X509Builder {
         let key = PubKey::Ec(EcPub {
-            sign_oid: sign_oid,
-            pub_oid: pub_oid,
+            pub_oid: oid,
             key: key,
             curve: curve,
         });
@@ -431,6 +533,11 @@ impl X509Builder {
         self
     }
 
+    pub fn pub_key_der(mut self, der: &Vec<u8>) -> X509Builder {
+        self.pub_key = get_pub_der(der);
+        self
+    }
+
     pub fn ext(mut self, ext: X509Ext) -> X509Builder {
         self.ext.push(ext);
         self
@@ -446,6 +553,11 @@ impl X509Builder {
         self
     }
 
+    pub fn sign_oid(mut self, oid: Vec<u64>) -> X509Builder {
+        self.sign_oid = oid;
+        self
+    }
+
     pub fn build(self) -> X509 {
         X509 {
             version: self.version,
@@ -456,6 +568,7 @@ impl X509Builder {
             not_after: self.not_after,
             pub_key: self.pub_key,
             ext: self.ext,
+            sign_oid: self.sign_oid,
             sign: Vec::new(),
         }
     }
@@ -477,10 +590,10 @@ fn get_asn1_set(v: &Vec<ASN1Block>, idx: usize) -> Option<&Vec<ASN1Block>> {
     }
 }
 
-fn get_asn1_uint64(v: &Vec<ASN1Block>, idx: usize) -> Option<u64> {
+fn get_serial_number(v: &Vec<ASN1Block>, idx: usize) -> Option<Vec<u8>> {
     let block = v.get(idx)?;
     match block {
-        ASN1Block::Integer(_, b) => b.to_u64(),
+        ASN1Block::Integer(_, b) => Some(BigInt::to_signed_bytes_be(b)),
         _ => None,
     }
 }
@@ -531,6 +644,14 @@ fn get_x509_name(v: &Vec<ASN1Block>, idx: usize) -> Option<Vec<X509Name>> {
                 oid: oid,
                 data: d.to_string(),
             }),
+            ASN1Block::TeletexString(_, d) => X509Name::TtxStr(OidStr {
+                oid: oid,
+                data: d.to_string(),
+            }),
+            ASN1Block::IA5String(_, d) => X509Name::Ia5Str(OidStr {
+                oid: oid,
+                data: d.to_string(),
+            }),
             _ => return None,
         };
 
@@ -540,7 +661,7 @@ fn get_x509_name(v: &Vec<ASN1Block>, idx: usize) -> Option<Vec<X509Name>> {
     Some(ret)
 }
 
-fn get_rsa_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Option<PubKey> {
+fn get_rsa_pub_key(v: &Vec<ASN1Block>, idx: usize) -> Option<PubKey> {
     let vec = get_asn1_seq(v, idx)?;
     let seq = get_asn1_seq(vec, 0)?;
     let pub_oid = match seq.get(0)? {
@@ -565,7 +686,6 @@ fn get_rsa_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Optio
     };
 
     let pub_key = PubKey::Rsa(RsaPub {
-        sign_oid: sign_oid.to_vec(),
         pub_oid: pub_oid,
         n: n,
         e: e,
@@ -574,7 +694,7 @@ fn get_rsa_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Optio
     Some(pub_key)
 }
 
-fn get_ec_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Option<PubKey> {
+fn get_ec_pub_key(v: &Vec<ASN1Block>, idx: usize) -> Option<PubKey> {
     let vec = get_asn1_seq(v, idx)?;
     let seq = get_asn1_seq(vec, 0)?;
     let pub_oid = match seq.get(0)? {
@@ -593,7 +713,6 @@ fn get_ec_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Option
     };
 
     let pub_key = PubKey::Ec(EcPub {
-        sign_oid: sign_oid.to_vec(),
         pub_oid: pub_oid,
         key: key.to_vec(),
         curve: curve,
@@ -602,16 +721,39 @@ fn get_ec_pub_key(v: &Vec<ASN1Block>, idx: usize, sign_oid: &Vec<u64>) -> Option
     Some(pub_key)
 }
 
-fn get_x509_time(v: &Vec<ASN1Block>, idx: usize) -> Option<(i64, i64)> {
+fn get_pub_key(v: &Vec<ASN1Block>, idx: usize) -> Option<PubKey> {
+    let mut pub_key = get_rsa_pub_key(v, idx);
+    if pub_key == None {
+        pub_key = get_ec_pub_key(v, idx);
+    }
+
+    pub_key
+}
+
+fn get_pub_der(der: &Vec<u8>) -> Option<PubKey> {
+    let asn = match simple_asn1::from_der(der) {
+        Ok(a) => a,
+        Err(_) => {
+            println!("Failed to deserialize Public Key");
+            return None;
+        }
+    };
+
+    get_pub_key(&asn, 0)
+}
+
+fn get_x509_time(v: &Vec<ASN1Block>, idx: usize) -> Option<(X509Time, X509Time)> {
     let vec = get_asn1_seq(v, idx)?;
 
     let not_before = match vec.get(0)? {
-        ASN1Block::UTCTime(_, t) => DateTime::<Utc>::timestamp(t),
+        ASN1Block::UTCTime(_, t) => X509Time::Utc(DateTime::<Utc>::timestamp(t)),
+        ASN1Block::GeneralizedTime(_, t) => X509Time::Gen(DateTime::<Utc>::timestamp(t)),
         _ => return None,
     };
 
     let not_after = match vec.get(1)? {
-        ASN1Block::UTCTime(_, t) => DateTime::<Utc>::timestamp(t),
+        ASN1Block::UTCTime(_, t) => X509Time::Utc(DateTime::<Utc>::timestamp(t)),
+        ASN1Block::GeneralizedTime(_, t) => X509Time::Gen(DateTime::<Utc>::timestamp(t)),
         _ => return None,
     };
 
@@ -629,18 +771,23 @@ fn get_ext_raw(v: &Vec<ASN1Block>) -> Option<Vec<X509Ext>> {
         };
 
         let critical = match seq.get(1)? {
-            ASN1Block::Boolean(_, c) => c,
-            _ => return None,
+            ASN1Block::Boolean(_, c) => *c,
+            _ => false,
         };
 
-        let data = match seq.get(2)? {
+        let idx = match critical {
+            false => 1,
+            true => 2,
+        };
+
+        let data = match seq.get(idx)? {
             ASN1Block::OctetString(_, d) => d,
             _ => return None,
         };
 
         ret.push(X509Ext {
             oid: oid,
-            critical: *critical,
+            critical: critical,
             data: data.to_vec(),
         });
     }
@@ -714,7 +861,7 @@ impl X509Deserialize for Vec<u8> {
         };
 
         /* Serial Number */
-        let sn = match get_asn1_uint64(body, idx) {
+        let sn = match get_serial_number(body, idx) {
             Some(a) => a,
             None => {
                 println!("Failed to get SerialNumber");
@@ -764,14 +911,13 @@ impl X509Deserialize for Vec<u8> {
         idx += 1;
 
         /* Subject Public Key Info */
-        let mut pub_key = get_rsa_pub_key(body, idx, &sign_oid);
-        if pub_key == None {
-            pub_key = get_ec_pub_key(body, idx, &sign_oid);
-        }
-        if pub_key == None {
-            println!("Failed to get Subject Public Key");
-            return None;
-        }
+        let pub_key = match get_pub_key(body, idx) {
+            Some(a) => a,
+            None => {
+                println!("Failed to get Subject Public Key");
+                return None;
+            }
+        };
         idx += 1;
 
         /* Extensions */
@@ -797,10 +943,11 @@ impl X509Deserialize for Vec<u8> {
             sn: sn,
             issuer: issuer,
             subject: subject,
-            not_before: not_before,
-            not_after: not_after,
-            pub_key: pub_key,
+            not_before: Some(not_before),
+            not_after: Some(not_after),
+            pub_key: Some(pub_key),
             ext: ext,
+            sign_oid: sign_oid,
             sign: sign,
         };
 
