@@ -1,10 +1,16 @@
+use regex::Regex;
 use ring::rand;
 use ring::signature::{self};
+use rustc_serialize::base64::FromBase64;
 use simple_x509::*;
+use std::fs;
 use std::fs::File;
 use std::io::prelude::*;
 use std::io::BufWriter;
 use std::io::Read;
+use std::str;
+
+const REGEX: &'static str = r"(-----BEGIN .*-----\n)((?:(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)*\n)+)(-----END .*-----)";
 
 fn read_file(f: &str) -> std::io::Result<Vec<u8>> {
     let mut fd = File::open(f)?;
@@ -45,6 +51,13 @@ fn ec_sign_fn(data: Vec<u8>) -> Option<Vec<u8>> {
     Some(sig.as_ref().to_vec())
 }
 
+pub fn pem_to_der(f: &str) -> Option<Vec<u8>> {
+    let r = Regex::new(REGEX).ok()?;
+    let v = r.replace(f, "$2");
+    let b = v.replace("\n", "");
+    b.from_base64().ok()
+}
+
 #[test]
 fn x509_rsa_root_test() {
     let pub_rsa_n: Vec<u8> = vec![
@@ -72,7 +85,7 @@ fn x509_rsa_root_test() {
     let state = "Some-State";
     let organization = "Internet Widgits Pty Ltd";
 
-    let x = X509Builder::new(0xf2f9d803d7b7d734)
+    let x = X509Builder::new(vec![0xf2, 0xf9, 0xd8, 0x03, 0xd7, 0xb7, 0xd7, 0x34])
         .version(2)
         .issuer_prstr(vec![2, 5, 4, 6], country) /* countryName */
         .issuer_utf8(vec![2, 5, 4, 8], state) /* stateOrProvinceName */
@@ -80,14 +93,14 @@ fn x509_rsa_root_test() {
         .subject_prstr(vec![2, 5, 4, 6], country) /* countryName */
         .subject_utf8(vec![2, 5, 4, 8], state) /* stateOrProvinceName */
         .subject_utf8(vec![2, 5, 4, 10], organization) /* organizationName */
-        .not_before(1_619_014_703)
-        .not_after(1_650_550_703)
+        .not_before_utc(1_619_014_703)
+        .not_after_utc(1_650_550_703)
         .pub_key_rsa(
-            vec![1, 2, 840, 113549, 1, 1, 11], /* sha256WithRSAEncryption (PKCS #1) */
-            vec![1, 2, 840, 113549, 1, 1, 1],  /* rsaEncryption (PKCS #1) */
+            vec![1, 2, 840, 113549, 1, 1, 1], /* rsaEncryption (PKCS #1) */
             pub_rsa_n,
             65537,
         )
+        .sign_oid(vec![1, 2, 840, 113549, 1, 1, 11]) /* sha256WithRSAEncryption (PKCS #1) */
         .build();
 
     let cert = match x.sign(|c| rsa_sign_fn(c)) {
@@ -104,6 +117,51 @@ fn x509_rsa_root_test() {
     assert_eq!(err, Ok(()));
 }
 
+#[test]
+fn x509_rsa_pub_key_test() {
+    let country = "AU";
+    let state = "Some-State";
+    let organization = "Internet Widgits Pty Ltd";
+    let pub_key = read_file("tests/data/rsa_pub.der").unwrap_or_else(|_| panic!("File not found"));
+
+    let x = X509Builder::new(vec![0xf2, 0xf9, 0xd8, 0x03, 0xd7, 0xb7, 0xd7, 0x34])
+        .version(2)
+        .issuer_prstr(vec![2, 5, 4, 6], country) /* countryName */
+        .issuer_utf8(vec![2, 5, 4, 8], state) /* stateOrProvinceName */
+        .issuer_utf8(vec![2, 5, 4, 10], organization) /* organizationName */
+        .subject_prstr(vec![2, 5, 4, 6], country) /* countryName */
+        .subject_utf8(vec![2, 5, 4, 8], state) /* stateOrProvinceName */
+        .subject_utf8(vec![2, 5, 4, 10], organization) /* organizationName */
+        .not_before_utc(1_619_014_703)
+        .not_after_utc(1_650_550_703)
+        .pub_key_der(&pub_key)
+        .sign_oid(vec![1, 2, 840, 113549, 1, 1, 11]) /* sha256WithRSAEncryption (PKCS #1) */
+        .build();
+
+    let cert = match x.sign(|c| rsa_sign_fn(c)) {
+        Some(c) => c,
+        None => panic!("sign() failed"),
+    };
+
+    let der = match cert.x509_enc() {
+        Some(d) => d,
+        None => panic!("x509_enc() failed"),
+    };
+
+    let err = write_file("tests/data/ca_rsa_pd.der", &der).map_err(|e| e.kind());
+    assert_eq!(err, Ok(()));
+
+    let x2 = der
+        .x509_dec()
+        .unwrap_or_else(|| panic!("Failed to deserialize"));
+
+    let pub_key2 = x2
+        .pub_key()
+        .unwrap_or_else(|| panic!("Failed to get pub_key"));
+
+    assert_eq!(pub_key, pub_key2);
+}
+
 #[test]
 fn x509_ec_root_test() {
     let pub_ec_key: Vec<u8> = vec![
@@ -118,7 +176,7 @@ fn x509_ec_root_test() {
     let state = "Some-State";
     let organization = "Internet Widgits Pty Ltd";
 
-    let x = X509Builder::new(0xf2f9d803d7b7d734)
+    let x = X509Builder::new(vec![0xf2, 0xf9, 0xd8, 0x03, 0xd7, 0xb7, 0xd7, 0x34])
         .version(2)
         .issuer_prstr(vec![ 2, 5, 4, 6 ], country) /* countryName */
         .issuer_utf8(vec![ 2, 5, 4, 8 ], state) /* stateOrProvinceName */
@@ -126,14 +184,14 @@ fn x509_ec_root_test() {
         .subject_prstr(vec![ 2, 5, 4, 6 ], country) /* countryName */
         .subject_utf8(vec![ 2, 5, 4, 8 ], state) /* stateOrProvinceName */
         .subject_utf8(vec![ 2, 5, 4, 10 ], organization) /* organizationName */
-        .not_before(1_619_014_703)
-        .not_after(1_650_550_703)
+        .not_before_utc(1_619_014_703)
+        .not_after_utc(1_650_550_703)
         .pub_key_ec(
-            vec![ 1, 2, 840, 10045, 4, 3, 2 ], /* ecdsaWithSHA256 (ANSI X9.62 ECDSA algorithm with SHA256) */
             vec![ 1, 2, 840, 10045, 2, 1 ], /* ecPublicKey (ANSI X9.62 public key type) */
             pub_ec_key,
             vec![ 1, 2, 840, 10045, 3, 1, 7 ], /* prime256v1 (ANSI X9.62 named elliptic curve) */
         )
+        .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
     let cert = match x.sign(|c| ec_sign_fn(c)) {
@@ -162,13 +220,12 @@ fn x509_extension_raw() {
 
     let common_name = "Name name";
 
-    let x = X509Builder::new(0xf2f9d803d7b7d734)
+    let x = X509Builder::new(vec![0xf2, 0xf9, 0xd8, 0x03, 0xd7, 0xb7, 0xd7, 0x34])
         .issuer_utf8(vec![ 2, 5, 4, 3 ], common_name) /* organizationName */
         .subject_utf8(vec![ 2, 5, 4, 3 ], common_name) /* organizationName */
-        .not_before(1_619_014_703)
-        .not_after(1_650_550_703)
+        .not_before_utc(1_619_014_703)
+        .not_after_utc(1_650_550_703)
         .pub_key_ec(
-            vec![ 1, 2, 840, 10045, 4, 3, 2 ], /* ecdsaWithSHA256 (ANSI X9.62 ECDSA algorithm with SHA256) */
             vec![ 1, 2, 840, 10045, 2, 1 ], /* ecPublicKey (ANSI X9.62 public key type) */
             pub_ec_key,
             vec![ 1, 2, 840, 10045, 3, 1, 7 ], /* prime256v1 (ANSI X9.62 named elliptic curve) */
@@ -178,6 +235,7 @@ fn x509_extension_raw() {
             true,
             vec![ 0x03, 0x02, 0x04, 0xB0 ],
         )
+        .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
     let cert = match x.sign(|c| ec_sign_fn(c)) {
@@ -214,18 +272,18 @@ fn x509_key_usage_extension() {
         ])
         .build();
 
-    let x = X509Builder::new(0xf2f9d803d7b7d734)
+    let x = X509Builder::new(vec![0xf2, 0xf9, 0xd8, 0x03, 0xd7, 0xb7, 0xd7, 0x34])
         .issuer_utf8(vec![ 2, 5, 4, 3 ], common_name) /* commonName */
         .subject_utf8(vec![ 2, 5, 4, 3 ], common_name) /* commonName */
-        .not_before(1_619_014_703)
-        .not_after(1_650_550_703)
+        .not_before_utc(1_619_014_703)
+        .not_after_utc(1_650_550_703)
         .pub_key_ec(
-            vec![ 1, 2, 840, 10045, 4, 3, 2 ], /* ecdsaWithSHA256 (ANSI X9.62 ECDSA algorithm with SHA256) */
             vec![ 1, 2, 840, 10045, 2, 1 ], /* ecPublicKey (ANSI X9.62 public key type) */
             pub_ec_key,
             vec![ 1, 2, 840, 10045, 3, 1, 7 ], /* prime256v1 (ANSI X9.62 named elliptic curve) */
         )
         .ext(key_usage)
+        .sign_oid(vec![ 1, 2, 840, 10045, 4, 3, 2 ]) /* ecdsaWithSHA256 (ANSI X9.62 ECDSA with SHA256) */
         .build();
 
     let cert = match x.sign(|c| ec_sign_fn(c)) {
@@ -267,3 +325,50 @@ fn x509_ec_deserialize() {
         .unwrap_or_else(|| panic!("Failed to serialize"));
     assert_eq!(der, der2);
 }
+
+#[test]
+fn x509_decoding_encoding() {
+    let ret = fs::read_dir("/etc/ssl/certs/");
+    let paths = match ret {
+        Ok(p) => p,
+        Err(_) => return, /* skip test */
+    };
+
+    let mut counter = 0;
+
+    for path in paths {
+        let p = match path {
+            Ok(pt) => pt.path().display().to_string(),
+            Err(_) => continue,
+        };
+        let pem = match read_file(&p) {
+            Ok(p) => p,
+            Err(_) => continue,
+        };
+        let st = match str::from_utf8(&pem) {
+            Ok(s) => s,
+            Err(_) => continue,
+        };
+        let der = match pem_to_der(st) {
+            Some(d) => d,
+            None => continue,
+        };
+
+        let x = match der.x509_dec() {
+            Some(x) => x,
+            None => {
+                println!("Failed to deserialize: {}", p);
+                continue;
+            }
+        };
+
+        let der2 = x
+            .x509_enc()
+            .unwrap_or_else(|| panic!("Failed to serialize: {}", p));
+
+        assert_eq!(der, der2);
+        counter += 1;
+    }
+
+    println!("{} certificates are tested", counter);
+}
